@@ -1,8 +1,5 @@
-# #!/usr/bin/env python
-# -*- coding: utf-8 -*-
-
 # <HTTPretty - HTTP client mock for Python>
-# Copyright (C) <2011-2018>  Gabriel Falcão <gabriel@nacaolivre.org>
+# Copyright (C) <2011-2021> Gabriel Falcão <gabriel@nacaolivre.org>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -25,31 +22,24 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 # OTHER DEALINGS IN THE SOFTWARE.
 
-from __future__ import unicode_literals
-
 import os
 import re
 import json
 import requests
-from sure import within, microseconds, expect
+import signal
+import httpretty
+
+from freezegun import freeze_time
+from contextlib import contextmanager
+from sure import within, miliseconds, expect
 from tornado import version as tornado_version
 from httpretty import HTTPretty, httprettified
-from httpretty.compat import text_type, PY3
 from httpretty.core import decode_utf8
 
-
-try:
-    from unittest import skip
-except ImportError:
-    from unittest2 import skip
-
 from tests.functional.base import FIXTURE_FILE, use_tornado_server
 
+from tests.compat import Mock
 
-if PY3:
-    xrange = range
-else:
-    pass
 
 try:
     advance_iterator = next
@@ -64,7 +54,7 @@ server_url = lambda path, port: "http://localhost:{}/{}".format(port, path.lstri
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_httpretty_should_mock_a_simple_get_with_requests_read(now):
     "HTTPretty should mock a simple GET with requests.get"
 
@@ -78,7 +68,21 @@ def test_httpretty_should_mock_a_simple_get_with_requests_read(now):
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
+def test_hostname_case_insensitive(now):
+    "HTTPretty should match the hostname case insensitive"
+
+    HTTPretty.register_uri(HTTPretty.GET, "http://yipit/",
+                           body="Find the best daily deals")
+
+    response = requests.get('http://YIPIT')
+    expect(response.text).to.equal('Find the best daily deals')
+    expect(HTTPretty.last_request.method).to.equal('GET')
+    expect(HTTPretty.last_request.path).to.equal('/')
+
+
+@httprettified
+@within(two=miliseconds)
 def test_httpretty_provides_easy_access_to_querystrings(now):
     "HTTPretty should provide an easy access to the querystring"
 
@@ -93,8 +97,8 @@ def test_httpretty_provides_easy_access_to_querystrings(now):
 
 
 @httprettified
-@within(two=microseconds)
-def test_httpretty_should_mock_headers_requests(now):
+@freeze_time("2013-10-04 04:20:00")
+def test_httpretty_should_mock_headers_requests():
     "HTTPretty should mock basic headers with requests"
 
     HTTPretty.register_uri(HTTPretty.GET, "http://github.com/",
@@ -110,13 +114,13 @@ def test_httpretty_should_mock_headers_requests(now):
         'content-length': '35',
         'status': '201',
         'server': 'Python/HTTPretty',
-        'date': now.strftime('%a, %d %b %Y %H:%M:%S GMT'),
+        'date': 'Fri, 04 Oct 2013 04:20:00 GMT',
     })
 
 
 @httprettified
-@within(two=microseconds)
-def test_httpretty_should_allow_adding_and_overwritting_requests(now):
+@freeze_time("2013-10-04 04:20:00")
+def test_httpretty_should_allow_adding_and_overwritting_requests():
     "HTTPretty should allow adding and overwritting headers with requests"
 
     HTTPretty.register_uri(HTTPretty.GET, "http://github.com/foo",
@@ -135,12 +139,12 @@ def test_httpretty_should_allow_adding_and_overwritting_requests(now):
         'content-length': '27',
         'status': '200',
         'server': 'Apache',
-        'date': now.strftime('%a, %d %b %Y %H:%M:%S GMT'),
+        'date': 'Fri, 04 Oct 2013 04:20:00 GMT',
     })
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_httpretty_should_allow_forcing_headers_requests(now):
     "HTTPretty should allow forcing headers with requests"
 
@@ -160,8 +164,8 @@ def test_httpretty_should_allow_forcing_headers_requests(now):
 
 
 @httprettified
-@within(two=microseconds)
-def test_httpretty_should_allow_adding_and_overwritting_by_kwargs_u2(now):
+@freeze_time("2013-10-04 04:20:00")
+def test_httpretty_should_allow_adding_and_overwritting_by_kwargs_u2():
     "HTTPretty should allow adding and overwritting headers by keyword args " \
         "with requests"
 
@@ -179,12 +183,12 @@ def test_httpretty_should_allow_adding_and_overwritting_by_kwargs_u2(now):
         'content-length': '27',
         'status': '200',
         'server': 'Apache',
-        'date': now.strftime('%a, %d %b %Y %H:%M:%S GMT'),
+        'date': 'Fri, 04 Oct 2013 04:20:00 GMT',
     })
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_rotating_responses_with_requests(now):
     "HTTPretty should support rotating responses with requests"
 
@@ -215,7 +219,7 @@ def test_rotating_responses_with_requests(now):
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_can_inspect_last_request(now):
     "HTTPretty.last_request is a mimetools.Message request from last match"
 
@@ -241,7 +245,7 @@ def test_can_inspect_last_request(now):
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_can_inspect_last_request_with_ssl(now):
     "HTTPretty.last_request is recorded even when mocking 'https' (SSL)"
 
@@ -267,7 +271,7 @@ def test_can_inspect_last_request_with_ssl(now):
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_httpretty_ignores_querystrings_from_registered_uri(now):
     "HTTPretty should ignore querystrings from the registered uri (requests library)"
 
@@ -281,13 +285,12 @@ def test_httpretty_ignores_querystrings_from_registered_uri(now):
 
 
 @httprettified
-@within(five=microseconds)
+@within(five=miliseconds)
 def test_streaming_responses(now):
     """
     Mock a streaming HTTP response, like those returned by the Twitter streaming
     API.
     """
-    from contextlib import contextmanager
 
     @contextmanager
     def in_time(time, message):
@@ -296,8 +299,6 @@ def test_streaming_responses(now):
         (unlike the `@within` decorator, which only complains afterward), or
         raise an AssertionError.
         """
-        import signal
-
         def handler(signum, frame):
             raise AssertionError(message)
         signal.signal(signal.SIGALRM, handler)
@@ -319,17 +320,21 @@ def test_streaming_responses(now):
                            streaming=True)
 
     # taken from the requests docs
+
+    # test iterating by line
     # Http://docs.python-requests.org/en/latest/user/advanced/# streaming-requests
     response = requests.post(TWITTER_STREAMING_URL, data={'track': 'requests'},
                              auth=('username', 'password'), stream=True)
 
-    # test iterating by line
     line_iter = response.iter_lines()
     with in_time(0.01, 'Iterating by line is taking forever!'):
-        for i in xrange(len(twitter_response_lines)):
+        for i in range(len(twitter_response_lines)):
             expect(next(line_iter).strip()).to.equal(
                 twitter_response_lines[i].strip())
 
+    HTTPretty.register_uri(HTTPretty.POST, TWITTER_STREAMING_URL,
+                           body=(l for l in twitter_response_lines),
+                           streaming=True)
     # test iterating by line after a second request
     response = requests.post(
         TWITTER_STREAMING_URL,
@@ -343,10 +348,13 @@ def test_streaming_responses(now):
     line_iter = response.iter_lines()
     with in_time(0.01, 'Iterating by line is taking forever the second time '
                        'around!'):
-        for i in xrange(len(twitter_response_lines)):
+        for i in range(len(twitter_response_lines)):
             expect(next(line_iter).strip()).to.equal(
                 twitter_response_lines[i].strip())
 
+    HTTPretty.register_uri(HTTPretty.POST, TWITTER_STREAMING_URL,
+                           body=(l for l in twitter_response_lines),
+                           streaming=True)
     # test iterating by char
     response = requests.post(
         TWITTER_STREAMING_URL,
@@ -364,7 +372,9 @@ def test_streaming_responses(now):
     expect(twitter_body).to.equal(twitter_expected_response_body)
 
     # test iterating by chunks larger than the stream
-
+    HTTPretty.register_uri(HTTPretty.POST, TWITTER_STREAMING_URL,
+                           body=(l for l in twitter_response_lines),
+                           streaming=True)
     response = requests.post(TWITTER_STREAMING_URL, data={'track': 'requests'},
                              auth=('username', 'password'), stream=True)
 
@@ -377,7 +387,7 @@ def test_streaming_responses(now):
 
 @httprettified
 def test_multiline():
-    url = 'http://httpbin.org/post'
+    url = 'https://httpbin.org/post'
     data = b'content=Im\r\na multiline\r\n\r\nsentence\r\n'
     headers = {
         'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
@@ -391,16 +401,18 @@ def test_multiline():
 
     expect(response.status_code).to.equal(200)
     expect(HTTPretty.last_request.method).to.equal('POST')
+    expect(HTTPretty.last_request.url).to.equal('https://httpbin.org/post')
+    expect(HTTPretty.last_request.protocol).to.equal('https')
     expect(HTTPretty.last_request.path).to.equal('/post')
     expect(HTTPretty.last_request.body).to.equal(data)
     expect(HTTPretty.last_request.headers['content-length']).to.equal('37')
     expect(HTTPretty.last_request.headers['content-type']).to.equal('application/x-www-form-urlencoded; charset=utf-8')
-    expect(len(HTTPretty.latest_requests)).to.equal(1)
+    expect(len(HTTPretty.latest_requests)).to.equal(2)
 
 
 @httprettified
 def test_octet_stream():
-    url = 'http://httpbin.org/post'
+    url = 'https://httpbin.org/post'
     data = b"\xf5\x00\x00\x00"  # utf-8 with invalid start byte
     headers = {
         'Content-Type': 'application/octet-stream',
@@ -413,16 +425,18 @@ def test_octet_stream():
 
     expect(response.status_code).to.equal(200)
     expect(HTTPretty.last_request.method).to.equal('POST')
+    expect(HTTPretty.last_request.url).to.equal('https://httpbin.org/post')
+    expect(HTTPretty.last_request.protocol).to.equal('https')
     expect(HTTPretty.last_request.path).to.equal('/post')
     expect(HTTPretty.last_request.body).to.equal(data)
     expect(HTTPretty.last_request.headers['content-length']).to.equal('4')
     expect(HTTPretty.last_request.headers['content-type']).to.equal('application/octet-stream')
-    expect(len(HTTPretty.latest_requests)).to.equal(1)
+    expect(len(HTTPretty.latest_requests)).to.equal(2)
 
 
 @httprettified
 def test_multipart():
-    url = 'http://httpbin.org/post'
+    url = 'https://httpbin.org/post'
     data = b'--xXXxXXyYYzzz\r\nContent-Disposition: form-data; name="content"\r\nContent-Type: text/plain; charset=utf-8\r\nContent-Length: 68\r\n\r\nAction: comment\nText: Comment with attach\nAttachment: x1.txt, x2.txt\r\n--xXXxXXyYYzzz\r\nContent-Disposition: form-data; name="attachment_2"; filename="x.txt"\r\nContent-Type: text/plain\r\nContent-Length: 4\r\n\r\nbye\n\r\n--xXXxXXyYYzzz\r\nContent-Disposition: form-data; name="attachment_1"; filename="x.txt"\r\nContent-Type: text/plain\r\nContent-Length: 4\r\n\r\nbye\n\r\n--xXXxXXyYYzzz--\r\n'
     headers = {'Content-Length': '495', 'Content-Type': 'multipart/form-data; boundary=xXXxXXyYYzzz', 'Accept': 'text/plain'}
     HTTPretty.register_uri(
@@ -432,15 +446,17 @@ def test_multipart():
     response = requests.post(url, data=data, headers=headers)
     expect(response.status_code).to.equal(200)
     expect(HTTPretty.last_request.method).to.equal('POST')
+    expect(HTTPretty.last_request.url).to.equal('https://httpbin.org/post')
+    expect(HTTPretty.last_request.protocol).to.equal('https')
     expect(HTTPretty.last_request.path).to.equal('/post')
     expect(HTTPretty.last_request.body).to.equal(data)
     expect(HTTPretty.last_request.headers['content-length']).to.equal('495')
     expect(HTTPretty.last_request.headers['content-type']).to.equal('multipart/form-data; boundary=xXXxXXyYYzzz')
-    expect(len(HTTPretty.latest_requests)).to.equal(1)
+    expect(len(HTTPretty.latest_requests)).to.equal(2)
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_callback_response(now):
     ("HTTPretty should call a callback function and set its return value as the body of the response"
      " requests")
@@ -469,7 +485,7 @@ def test_callback_response(now):
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_callback_body_remains_callable_for_any_subsequent_requests(now):
     ("HTTPretty should call a callback function more than one"
      " requests")
@@ -489,7 +505,7 @@ def test_callback_body_remains_callable_for_any_subsequent_requests(now):
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_callback_setting_headers_and_status_response(now):
     ("HTTPretty should call a callback function and uses it retur tuple as status code, headers and body"
      " requests")
@@ -540,7 +556,7 @@ def test_httpretty_should_respect_matcher_priority():
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_callback_setting_content_length_on_head(now):
     ("HTTPretty should call a callback function, use it's return tuple as status code, headers and body"
      " requests and respect the content-length header when responding to HEAD")
@@ -564,7 +580,7 @@ def test_httpretty_should_allow_registering_regexes_and_give_a_proper_match_to_t
 
     HTTPretty.register_uri(
         HTTPretty.GET,
-        re.compile("https://api.yipit.com/v1/deal;brand=(?P<brand_name>\w+)"),
+        re.compile(r"https://api.yipit.com/v1/deal;brand=(?P<brand_name>\w+)"),
         body=lambda method, uri, headers: [200, headers, uri]
     )
 
@@ -581,7 +597,7 @@ def test_httpretty_should_allow_registering_regexes():
 
     HTTPretty.register_uri(
         HTTPretty.GET,
-        re.compile("https://api.yipit.com/v1/deal;brand=(?P<brand_name>\w+)"),
+        re.compile(r"https://api.yipit.com/v1/deal;brand=(?P<brand_name>\w+)"),
         body="Found brand",
     )
 
@@ -598,7 +614,7 @@ def test_httpretty_provides_easy_access_to_querystrings_with_regexes():
 
     HTTPretty.register_uri(
         HTTPretty.GET,
-        re.compile("https://api.yipit.com/v1/(?P<endpoint>\w+)/$"),
+        re.compile(r"https://api.yipit.com/v1/(?P<endpoint>\w+)/$"),
         body="Find the best daily deals"
     )
 
@@ -610,22 +626,30 @@ def test_httpretty_provides_easy_access_to_querystrings_with_regexes():
     })
 
 
-@httprettified
-@skip
+@httprettified(verbose=True)
 def test_httpretty_allows_to_chose_if_querystring_should_be_matched():
     "HTTPretty should provide a way to not match regexes that have a different querystring"
 
     HTTPretty.register_uri(
         HTTPretty.GET,
-        re.compile("https://example.org/(?P<endpoint>\w+)/$"),
+        "http://localhost:9090",
+    )
+    HTTPretty.register_uri(
+        HTTPretty.GET,
+        re.compile(r"http://localhost:9090/what/?$"),
         body="Nudge, nudge, wink, wink. Know what I mean?",
         match_querystring=True
     )
-
-    response = requests.get('https://example.org/what/')
+    HTTPretty.register_uri(
+        HTTPretty.GET,
+        re.compile(r"http://localhost:9090/what.*[?]?.*"),
+        body="Different",
+        match_querystring=False
+    )
+    response = requests.get('http://localhost:9090/what/')
     expect(response.text).to.equal('Nudge, nudge, wink, wink. Know what I mean?')
 
-    response = requests.get('https://example.org/what/?flying=coconuts')
+    response = requests.get('http://localhost:9090/what/', params={'flying': 'coconuts'})
     expect(response.text).to.not_be.equal('Nudge, nudge, wink, wink. Know what I mean?')
 
 
@@ -650,7 +674,7 @@ def test_httpretty_should_allow_multiple_methods_for_the_same_uri():
 @httprettified
 def test_httpretty_should_allow_registering_regexes_with_streaming_responses():
     "HTTPretty should allow registering regexes with streaming responses"
-    import os
+
     os.environ['DEBUG'] = 'true'
 
     def my_callback(request, url, headers):
@@ -659,7 +683,7 @@ def test_httpretty_should_allow_registering_regexes_with_streaming_responses():
 
     HTTPretty.register_uri(
         HTTPretty.POST,
-        re.compile("https://api.yipit.com/v1/deal;brand=(?P<brand_name>\w+)"),
+        re.compile(r"https://api.yipit.com/v1/deal;brand=(?P<brand_name>\w+)"),
         body=my_callback,
     )
 
@@ -784,8 +808,13 @@ def test_recording_calls(port):
         ]
     })
     response['response'].should.have.key("status").being.equal(200)
-    response['response'].should.have.key("body").being.an(text_type)
+    response['response'].should.have.key("body").being.an(str)
     response['response'].should.have.key("headers").being.a(dict)
+    # older urllib3 had a bug where header keys were lower-cased:
+    # https://github.com/shazow/urllib3/issues/236
+    # cope with that
+    if 'server' in response['response']["headers"]:
+        response['response']["headers"]["Server"] = response['response']["headers"].pop("server")
     response['response']["headers"].should.have.key("Server").being.equal("TornadoServer/" + tornado_version)
 
     # And When I playback the previously recorded calls
@@ -811,8 +840,6 @@ def test_py26_callback_response():
     ("HTTPretty should call a callback function *once* and set its return value"
      " as the body of the response requests")
 
-    from mock import Mock
-
     def _request_callback(request, uri, headers):
         return [200, headers, "The {} response from {}".format(decode_utf8(request.method), uri)]
 
@@ -837,7 +864,7 @@ def test_httpretty_should_work_with_non_standard_ports():
 
     HTTPretty.register_uri(
         HTTPretty.GET,
-        re.compile("https://api.yipit.com:1234/v1/deal;brand=(?P<brand_name>\w+)"),
+        re.compile(r"https://api.yipit.com:1234/v1/deal;brand=(?P<brand_name>\w+)"),
         body=lambda method, uri, headers: [200, headers, uri]
     )
     HTTPretty.register_uri(
@@ -896,7 +923,7 @@ def test_httpretty_should_allow_registering_regexes_with_port_and_give_a_proper_
 
     HTTPretty.register_uri(
         HTTPretty.GET,
-        re.compile("https://api.yipit.com:1234/v1/deal;brand=(?P<brand_name>\w+)"),
+        re.compile(r"https://api.yipit.com:1234/v1/deal;brand=(?P<brand_name>\w+)"),
         body=lambda method, uri, headers: [200, headers, uri]
     )
 
@@ -905,3 +932,18 @@ def test_httpretty_should_allow_registering_regexes_with_port_and_give_a_proper_
     expect(response.text).to.equal('https://api.yipit.com:1234/v1/deal;brand=gap?first_name=chuck&last_name=norris')
     expect(HTTPretty.last_request.method).to.equal('GET')
     expect(HTTPretty.last_request.path).to.equal('/v1/deal;brand=gap?first_name=chuck&last_name=norris')
+
+
+@httprettified
+def test_httpretty_should_handle_paths_starting_with_two_slashes():
+    "HTTPretty should handle URLs with paths starting with //"
+
+    HTTPretty.register_uri(
+        HTTPretty.GET, "http://example.com//foo",
+        body="Find the best foo"
+    )
+
+    response = requests.get('http://example.com//foo')
+    expect(response.text).to.equal('Find the best foo')
+    expect(HTTPretty.last_request.method).to.equal('GET')
+    expect(HTTPretty.last_request.path).to.equal('//foo')
