@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # <HTTPretty - HTTP client mock for Python>
-# Copyright (C) <2011-2018>  Gabriel Falcão <gabriel@nacaolivre.org>
+# Copyright (C) <2011-2021> Gabriel Falcão <gabriel@nacaolivre.org>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -34,13 +34,14 @@ except ImportError:
     import urllib2
     urlopen = urllib2.urlopen
 
-from sure import within, microseconds
+from freezegun import freeze_time
+from sure import within, miliseconds
 from httpretty import HTTPretty, httprettified
 from httpretty.core import decode_utf8
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_httpretty_should_mock_a_simple_get_with_urllib2_read():
     "HTTPretty should mock a simple GET with urllib2.read()"
 
@@ -55,7 +56,7 @@ def test_httpretty_should_mock_a_simple_get_with_urllib2_read():
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_httpretty_provides_easy_access_to_querystrings(now):
     "HTTPretty should provide an easy access to the querystring"
 
@@ -73,8 +74,8 @@ def test_httpretty_provides_easy_access_to_querystrings(now):
 
 
 @httprettified
-@within(two=microseconds)
-def test_httpretty_should_mock_headers_urllib2(now):
+@freeze_time("2013-10-04 04:20:00")
+def test_httpretty_should_mock_headers_urllib2():
     "HTTPretty should mock basic headers with urllib2"
 
     HTTPretty.register_uri(HTTPretty.GET, "http://github.com/",
@@ -93,13 +94,13 @@ def test_httpretty_should_mock_headers_urllib2(now):
         'content-length': '35',
         'status': '201',
         'server': 'Python/HTTPretty',
-        'date': now.strftime('%a, %d %b %Y %H:%M:%S GMT'),
+        'date': 'Fri, 04 Oct 2013 04:20:00 GMT',
     })
 
 
 @httprettified
-@within(two=microseconds)
-def test_httpretty_should_allow_adding_and_overwritting_urllib2(now):
+@freeze_time("2013-10-04 04:20:00")
+def test_httpretty_should_allow_adding_and_overwritting_urllib2():
     "HTTPretty should allow adding and overwritting headers with urllib2"
 
     HTTPretty.register_uri(HTTPretty.GET, "http://github.com/",
@@ -121,12 +122,12 @@ def test_httpretty_should_allow_adding_and_overwritting_urllib2(now):
         'content-length': '27',
         'status': '200',
         'server': 'Apache',
-        'date': now.strftime('%a, %d %b %Y %H:%M:%S GMT'),
+        'date': 'Fri, 04 Oct 2013 04:20:00 GMT',
     })
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_httpretty_should_allow_forcing_headers_urllib2():
     "HTTPretty should allow forcing headers with urllib2"
 
@@ -148,8 +149,8 @@ def test_httpretty_should_allow_forcing_headers_urllib2():
 
 
 @httprettified
-@within(two=microseconds)
-def test_httpretty_should_allow_adding_and_overwritting_by_kwargs_u2(now):
+@freeze_time("2013-10-04 04:20:00")
+def test_httpretty_should_allow_adding_and_overwritting_by_kwargs_u2():
     ("HTTPretty should allow adding and overwritting headers by "
      "keyword args with urllib2")
 
@@ -171,12 +172,12 @@ def test_httpretty_should_allow_adding_and_overwritting_by_kwargs_u2(now):
         'content-length': str(len(body)),
         'status': '200',
         'server': 'Apache',
-        'date': now.strftime('%a, %d %b %Y %H:%M:%S GMT'),
+        'date': 'Fri, 04 Oct 2013 04:20:00 GMT',
     })
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_httpretty_should_support_a_list_of_successive_responses_urllib2(now):
     ("HTTPretty should support adding a list of successive "
      "responses with urllib2")
@@ -209,7 +210,7 @@ def test_httpretty_should_support_a_list_of_successive_responses_urllib2(now):
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_can_inspect_last_request(now):
     "HTTPretty.last_request is a mimetools.Message request from last match"
 
@@ -238,7 +239,7 @@ def test_can_inspect_last_request(now):
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_can_inspect_last_request_with_ssl(now):
     "HTTPretty.last_request is recorded even when mocking 'https' (SSL)"
 
@@ -267,7 +268,7 @@ def test_can_inspect_last_request_with_ssl(now):
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_httpretty_ignores_querystrings_from_registered_uri():
     "HTTPretty should mock a simple GET with urllib2.read()"
 
@@ -284,9 +285,9 @@ def test_httpretty_ignores_querystrings_from_registered_uri():
 
 
 @httprettified
-@within(two=microseconds)
+@within(two=miliseconds)
 def test_callback_response(now):
-    ("HTTPretty should all a callback function to be set as the body with"
+    ("HTTPretty should call a callback function to be set as the body with"
      " urllib2")
 
     def request_callback(request, uri, headers):
@@ -326,7 +327,7 @@ def test_httpretty_should_allow_registering_regexes():
 
     HTTPretty.register_uri(
         HTTPretty.GET,
-        re.compile("https://api.yipit.com/v1/deal;brand=(?P<brand_name>\w+)"),
+        re.compile(r"https://api.yipit.com/v1/deal;brand=(?P<brand_name>\w+)"),
         body="Found brand",
     )
 
