@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # <HTTPretty - HTTP client mock for Python>
-# Copyright (C) <2011-2018>  Gabriel Falcão <gabriel@nacaolivre.org>
+# Copyright (C) <2011-2021> Gabriel Falcão <gabriel@nacaolivre.org>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -25,6 +25,7 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 # OTHER DEALINGS IN THE SOFTWARE.
 from __future__ import unicode_literals
+import re
 import json
 from sure import expect
 import httpretty
@@ -34,12 +35,8 @@ from httpretty import core
 from httpretty.core import URIInfo, BaseClass, Entry, FakeSockFile, HTTPrettyRequest
 from httpretty.http import STATUSES
 
-try:
-    from mock import MagicMock
-    from mock import patch
-except ImportError:
-    from unittest.mock import MagicMock
-    from unittest.mock import patch
+from tests.compat import MagicMock, patch
+
 
 TEST_HEADER = """
 GET /test/test.html HTTP/1.1
@@ -66,47 +63,6 @@ def test_httpretty_should_raise_proper_exception_on_inconsistent_length():
     )
 
 
-def test_httpretty_should_raise_on_socket_send_when_uri_registered():
-    ("HTTPretty should raise a RuntimeError when the fakesocket is "
-     "used in an invalid usage.")
-
-    import socket
-    HTTPretty.enable()
-
-    HTTPretty.register_uri(HTTPretty.GET,
-                           'http://127.0.0.1:5000')
-    expect(core.POTENTIAL_HTTP_PORTS).to.be.equal(set([80, 5000]))
-    expect(core.POTENTIAL_HTTPS_PORTS).to.be.equal(set([443]))
-
-    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-    sock.connect(('127.0.0.1', 5000))
-    expect(sock.send).when.called_with(b'whatever').to.throw(RuntimeError)
-    sock.close()
-
-    # restore the previous value
-    core.POTENTIAL_HTTP_PORTS.remove(5000)
-    HTTPretty.reset()
-    HTTPretty.disable()
-
-
-def test_httpretty_should_not_raise_on_socket_send_when_uri_not_registered():
-    ("HTTPretty should not raise a RuntimeError when the fakesocket "
-     "is used in an invalid usage.")
-
-    import socket
-    HTTPretty.enable()
-
-    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, 0)
-    sock.setblocking(0)
-    expect(sock.sendto).when.called_with(b'whatever',
-                                         ('127.0.0.1', 53)
-                                         ).should_not.throw(RuntimeError)
-
-    sock.close()
-    HTTPretty.reset()
-    HTTPretty.disable()
-
-
 def test_does_not_have_last_request_by_default():
     'HTTPretty.last_request is a dummy object by default'
     HTTPretty.reset()
@@ -208,7 +164,7 @@ def test_uri_info_full_url():
     )
 
     expect(uri_info.full_url()).to.equal(
-        "http://johhny:password@google.com/?foo=bar&baz=test"
+        "http://johhny:password@google.com/?baz=test&foo=bar"
     )
 
     expect(uri_info.full_url(use_querystring=False)).to.equal(
@@ -322,8 +278,8 @@ def test_fake_socket_passes_through_bind():
     HTTPretty.enable()
     s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     s.truesock = MagicMock()
-    expect(s.bind).called_with().should_not.throw(AttributeError)
-    s.truesock.bind.assert_called_with()
+    expect(s.bind).called_with(('127.0.0.1', 1000)).should_not.throw(AttributeError)
+    s.truesock.bind.assert_called_with(('127.0.0.1', 1000))
 
 
 def test_fake_socket_passes_through_connect_ex():
@@ -447,3 +403,27 @@ def test_socktype_good_python_version():
         HTTPretty.enable()
         expect(socket.SocketType).to.equal(socket.socket)
         HTTPretty.disable()
+
+
+def test_httpretty_should_allow_registering_regex_hostnames():
+    "HTTPretty should allow registering regexes with requests"
+
+    HTTPretty.register_uri(
+        HTTPretty.GET,
+        re.compile(r'^http://\w+\.foo\.com/baz$'),
+        body="yay",
+    )
+
+    assert HTTPretty.match_http_address('www.foo.com', 80)
+
+
+def test_httpretty_should_allow_registering_regex_hostnames_ssl():
+    "HTTPretty should allow registering regexes with requests (ssl version)"
+
+    HTTPretty.register_uri(
+        HTTPretty.GET,
+        re.compile(r'^https://\w+\.foo\.com/baz$'),
+        body="yay",
+    )
+
+    assert HTTPretty.match_https_hostname('www.foo.com')
