@@ -1,10 +1,14 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
-from mock import patch
 import httpretty
 from httpretty.core import HTTPrettyRequest
 
+try:
+    from unittest.mock import patch
+except ImportError:
+    from mock import patch
+
 
 @patch('httpretty.httpretty')
 def test_last_request(original):
@@ -13,6 +17,13 @@ def test_last_request(original):
     httpretty.last_request().should.equal(original.last_request)
 
 
+@patch('httpretty.httpretty')
+def test_latest_requests(original):
+    ("httpretty.latest_requests() should return httpretty.core.latest_requests")
+
+    httpretty.latest_requests().should.equal(original.latest_requests)
+
+
 def test_has_request():
     ("httpretty.has_request() correctly detects "
      "whether or not a request has been made")
